import os
import sys
import subprocess
import pstats
from tempfile import mkdtemp
from cProfile import Profile

BASE_DIR = os.path.abspath(os.path.dirname(__file__))
PROCESS_KILLER_PATH = os.path.join(BASE_DIR, '_process_killer.py')


def record_stats() -> Profile:
    profiler = Profile()
    profiler.enable()
    return profiler


def show_results(
        profile_file: str, 
        open_flamechart: bool, 
        print_stats: bool,
        verbose: bool) -> None:
    """Shows the profiling results, either as flamechart, and/or as text output"""
    if open_flamechart:
        print('Opening flamechart now.')
        # Start snakeviz to show the results
        process = subprocess.Popen(
            [sys.executable, '-m', 'snakeviz', profile_file], 
            stdout=None if verbose else subprocess.DEVNULL
        )

        # Shutdown snakeviz process after the browser had time to open the results
        subprocess.Popen(
            [sys.executable, PROCESS_KILLER_PATH, str(process.pid)]
        )
        
    # Print the profiling statistics
    if print_stats:
        stats = pstats.Stats(profile_file)
        stats.print_stats()


def get_profile_file_path() -> str:
    """Creates a temporary directory and returns the file for the profiling results"""
    temp_dir = mkdtemp()
    profile_file = f"{temp_dir}/profile.prof"
    return profile_file


from functools import wraps
from .utils import show_results, record_stats, get_profile_file_path


class profile:
    """
    A profiling decorator and context manager. It uses cProfile to profile the 
    execution of Python code and uses snakeviz to visualize the results in a 
    developer-friendly way.
    """
    def __init__(
            self, 
            open_flamechart=True,
            print_stats=False,
            verbose=False) -> None:
        self.profiler = None
        self.open_flamechart = open_flamechart
        self.print_stats = print_stats
        self.verbose = verbose

    def __call__(self, func):
        @wraps(func)
        def profiled_func(*args, **kwargs):
            self.profiler = record_stats()
            result = func(*args, **kwargs)
            self._dump_stats_and_analyze()
            return result

        return profiled_func

    def __enter__(self):
        self.profiler = record_stats()

    def __exit__(self, exc_type, exc_value, traceback):
        self._dump_stats_and_analyze()

    def _dump_stats_and_analyze(self):
        profile_file = get_profile_file_path()

        print(f'\nNew profiling result')
        print(f'Run "python -m snakeviz {profile_file}" to re-render '
                'the flame chart.')

        self.profiler.disable()
        self.profiler.dump_stats(profile_file)

        show_results(
            profile_file=profile_file,
            open_flamechart=self.open_flamechart,
            print_stats=self.print_stats,
            verbose=self.verbose
        )
        print('\n')


"""
Automatically shutdown running snakeviz instances after the browser opened
the flame chart.
"""

import os 
import sys
from time import sleep
from signal import SIGTERM


if __name__ == '__main__':
    if len(sys.argv) != 2:
        raise AssertionError('Please provide the process ID')
    try:
        pid = int(sys.argv[1])
    except ValueError:
        raise AssertionError(f'Invalid process ID "{sys.argv[1]}"')
    
    sleep(5)
    os.kill(pid, SIGTERM)


from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup( 
    name='whyslow', 
    version='0.1.4', 
    description='Developer-friendly performance profiling utilities for Python', 
    long_description=long_description,
    long_description_content_type="text/markdown",    
    author='valentinstn', 
    packages=['whyslow'], 
    python_requires='>=3.6',
    install_requires=[ 
        'snakeviz'
    ], 
    project_urls={
        "GitHub": "https://github.com/valentinstn/whyslow",
    },
) 



import sys 
import os
from subprocess import call
from tempfile import mkdtemp
from .utils import show_results, get_profile_file_path

if len(sys.argv) != 2:
    raise AssertionError('No file name provided to be executed')

profile_file = get_profile_file_path()
call([sys.executable, '-m', 'cProfile', '-o', profile_file, *sys.argv[1:]])

if os.path.exists(profile_file):
    show_results(
        profile_file=profile_file,
        open_flamechart=True,
        print_stats=False,
        verbose=False
    )
else:
    raise AssertionError('No profiling results')